@@ -20,6 +20,7 @@ __status__      = "rc1"
 __date__        = "2019-08-09"
 
 
+from warnings import warn
 # --------------------------------------------------------------------
 # create internal helper functions to be used for ALL sparql queries
 # --------------------------------------------------------------------
@@ -121,12 +122,13 @@ def objectSpec(spec='atcCo2L2DataObject', station='', limit=0):
 			FILTER NOT EXISTS {[] cpmeta:isNextVersionOf ?dobj}
 			?dobj cpmeta:wasSubmittedBy/prov:endedAtTime ?submEnd .
 			?dobj cpmeta:wasAcquiredBy [
-				prov:wasAssociatedWith/cpmeta:hasName ?station ;
+				prov:wasAssociatedWith ?stationIri ;
 				cpmeta:hasSamplingHeight ?samplingHeight
-			] .			
+			] .
+            ?stationIri cpmeta:hasName ?station .		
 			%s
 		}
-		order by ?station ?samplingHeight		
+		order by ?station ?samplingHeight
         %s
         """ % (spec,station, __checklimit__(limit))
 
@@ -315,48 +317,29 @@ def stationData(uri, level='2'):
 
     return query
 
-def stations_with_pi(station = '', limit=0):
-    """
-        Define SPARQL query to get a list of ICOS stations with PI and email.
-        As per writing, (April 2019, the list is pulled from the provisional
-        data, labeling process)
-    """
-    if station:
-        flt = 'FILTER(?stationId = "' + station + '") . '
-    else:
-        flt = station
 
-    query = """
-            prefix st: <http://meta.icos-cp.eu/ontologies/stationentry/>
-            select distinct ?stationId ?stationName ?stationTheme 
-            ?class  ?siteType
-            ?lat ?lon ?eas ?eag ?firstName ?lastName ?email ?country
-            from <http://meta.icos-cp.eu/resources/stationentry/>
-            where{
-                ?s st:hasShortName ?stationId .
-                %s
-                optional{?s st:hasLon ?lon} .
-                optional{?s st:hasLat ?lat} .
-                optional{?s st:hasElevationAboveSea ?eas} .
-                optional{?s st:hasElevationAboveGround ?eag} .                
-                ?s st:hasLongName ?stationName .
-                ?s st:hasPi ?pi .
-                ?pi st:hasFirstName ?firstName .
-                ?pi st:hasLastName ?lastName .
-                ?pi st:hasEmail ?email .
-                ?s a ?stationClass .
-                ?s st:hasStationClass ?class .
-                optional{?s st:hasCountry ?country} .
-                optional{?s st:hasSiteType ?siteType} .
-                
-                BIND (replace(str(?stationClass), "http://meta.icos-cp.eu/ontologies/stationentry/", "") AS ?stationTheme )
-            }            
-            %s
-        """ % (flt, __checklimit__(limit))
+def stations_with_pi(station='', limit=0):
+    """
+        Definition of an old SPARQL query that was used to fetch provisional
+        meta-data, namely a list of ICOS stations with principal
+        investigators and e-mails.
+    """
+    deprecation_message = """
+    This function will be deprecated in the next pylib major release.
+    In its current version the function fetches a query that will return all stations or a number of stations
+    given the `limit` argument.
+    In order to filter only stations with a principal investigator one can run:
+        from icoscp.sparql import sparqls
+        from icoscp.sparql.runsparql import RunSparql
+        query = sparqls.stations_with_pi()
+        df = RunSparql(sparql_query=query, output_format='pandas').run()
+        df_stations_with_pi = df[(df.firstName.notnull()) & (df.lastName.notnull())]
+    """
+    warn(deprecation_message, DeprecationWarning, stacklevel=2)
+    return getStations(station=station) + __checklimit__(limit)
 
-    return query
-# -----------------------------------------------------------------------------
-def getStations(station = ''):
+
+def getStations(station='', theme=''):
     """
     Define SPARQL query to return a list of all known stations
     at the Carbon Portal. This can include NON-ICOS stations.
@@ -365,92 +348,78 @@ def getStations(station = ''):
     Parameters
     ----------
     station : str, optional, case sensitive
-        DESCRIPTION. The default is '', and empyt string which returns ALL
+        DESCRIPTION. The default is '', and empty string which returns ALL
         stations. If you provide a station id, be aware, that it needs to be
-        exactly as provided from the Triple Store....case sensitive. 
+        exactly as provided from the Triple Store....case sensitive.
+    theme : str, optional, case sensitive
+        DESCRIPTION. The default is '', and empty string which returns ALL
+        themes. If you provide a theme, be aware, that it needs to be
+        exactly as provided from the Triple Store....case sensitive.
     Returns
     -------
     query : str, valid sparql query to run against the Carbon Portal SPARQL endpoint.
     """
     if station:
-        flt = 'FILTER(?id = "' + station + '") . '
+        station_filter = f'BIND ( "{station}"^^xsd:string as ?id)'
     else:
-        flt = station
-    
-    query = """
-            prefix cpmeta: <http://meta.icos-cp.eu/ontologies/cpmeta/>
-            select *
-            from <http://meta.icos-cp.eu/resources/icos/> 
-            from <http://meta.icos-cp.eu/resources/extrastations/> 
-            where {
-            	?uri cpmeta:hasStationId ?id .   
-                %s
-            	OPTIONAL {?uri cpmeta:hasName ?name  } .                
-            	OPTIONAL {?uri cpmeta:countryCode ?country }.
-            	OPTIONAL {?uri cpmeta:hasLatitude ?lat }.
-            	OPTIONAL {?uri cpmeta:hasLongitude ?lon }.
-            	OPTIONAL {?uri cpmeta:hasElevation ?elevation } .
-            }
-            """ %(flt)
-            
-            
-    return query
-
-# -----------------------------------------------------------------------------
-def cpbGetInfo(dobj):
-    """
-        Define SPARQL query to get information about a digitial object
-        aiming to assmble the underlying binary data structure schema for
-        a fast access to the binary file.
-        Input dobj: Persisten Identifier
-        dobj may be in the form of the PID or the full handle
-    """
-   
-    query = """
-            prefix cpmeta: <http://meta.icos-cp.eu/ontologies/cpmeta/>
-            select * where {
-                values ?dobj { %s }
-                ?dobj cpmeta:hasObjectSpec ?objSpec ;
-                cpmeta:hasNumberOfRows ?nRows ;
-                cpmeta:hasName ?fileName .
-                ?objSpec rdfs:label ?specLabel .
-                OPTIONAL{?dobj cpmeta:hasActualColumnNames ?columnNames }
-            }
-            
-            """ %__dobjUrl__(dobj)
-
-    return query
-# -----------------------------------------------------------------------------
-def cpbGetSchemaDetail(formatSpec):
-    """
-        Define SPARQL query to get information about a digitial object
-        to assmble the underlying binary data structure schema for
-        access to the binary file.
-        :param formatSpec url for the specification  
-    """
+        station_filter = ''
+    if theme:
+        theme_filter = f'BIND (cpmeta:{theme} as ?stationTheme)'
+    else:
+        theme_filter = ''
 
     query = """
-            prefix cpmeta: <http://meta.icos-cp.eu/ontologies/cpmeta/>
-            SELECT distinct ?objFormat ?colName ?valueType ?valFormat ?unit ?qKind ?colTip ?isRegex
-            WHERE {
-                <%(formSpec)s> cpmeta:containsDataset ?dset .
-                <%(formSpec)s> cpmeta:hasFormat ?objFormat .
-                ?dset cpmeta:hasColumn ?column .
-                ?column cpmeta:hasColumnTitle ?colName ;
-                    cpmeta:hasValueFormat ?valFormat ;
-                    cpmeta:hasValueType ?valType .                
-                ?valType rdfs:label ?valueType .
-                optional{?valType rdfs:comment ?colTip }
-                optional{ ?column cpmeta:isRegexColumn ?isRegex } .
-                optional{
-                    ?valType cpmeta:hasUnit ?unit .
-                    ?valType cpmeta:hasQuantityKind [rdfs:label ?qKind ] .
+        prefix xsd: <http://www.w3.org/2001/XMLSchema#>
+        prefix cpmeta: <http://meta.icos-cp.eu/ontologies/cpmeta/>
+        prefix cpst: <http://meta.icos-cp.eu/ontologies/stationentry/>
+        select ?uri ?id ?name ?icosClass ?country ?lat ?lon ?elevation ?stationTheme ?firstName ?lastName ?email ?siteType
+        from <http://meta.icos-cp.eu/resources/icos/>
+        from <http://meta.icos-cp.eu/resources/extrastations/>
+        from <http://meta.icos-cp.eu/resources/cpmeta/>
+        from <http://meta.icos-cp.eu/resources/stationentry/>
+        where {
+            {
+                select ?uri ?id ?stationTheme (sample(?pers) as ?piOpt)  where
+                {
+                    %s
+                    ?uri cpmeta:hasStationId ?id .
+                    %s
+                    ?uri a ?stationTheme .
+                    OPTIONAL{
+                        ?memb cpmeta:atOrganization ?uri ; cpmeta:hasRole <http://meta.icos-cp.eu/resources/roles/PI> .
+                        filter not exists {?memb cpmeta:hasEndTime []}
+                        ?pers cpmeta:hasMembership ?memb
+                    }
                 }
-            } order by ?colName                       
-            """ % {'formSpec': formatSpec}
+                group by ?uri ?id ?stationTheme
+            }
+            bind(coalesce(?piOpt, <http://dummy>) as ?pi)
+            OPTIONAL{
+                ?pi cpmeta:hasFirstName ?firstName .
+                ?pi cpmeta:hasLastName ?lastName
+            }
+            OPTIONAL{?pi cpmeta:hasEmail ?email}
+            OPTIONAL {
+                    {
+                        ?provSt cpst:hasProductionCounterpart ?prodUriLit .
+                        bind((?uri = iri(?prodUriLit)) as ?areEqual)
+                        filter(?areEqual)
+                    }
+                ?provSt cpst:hasSiteType ?siteType .
+            }
+            OPTIONAL {?uri cpmeta:hasStationClass ?icosClass}
+            OPTIONAL {?uri cpmeta:hasName ?name  }
+            OPTIONAL {?uri cpmeta:countryCode ?country }
+            OPTIONAL {?uri cpmeta:hasLatitude ?lat }
+            OPTIONAL {?uri cpmeta:hasLongitude ?lon }
+            OPTIONAL {?uri cpmeta:hasElevation ?elevation }
+        }
+        order by ?stationTheme ?id
+        """ % (station_filter, theme_filter)
+            
     return query
 
-# -----------------------------------------------------------------------------
+
 def dobjStation(dobj):
     """
         Define SPARQL query to get information about a station
@@ -519,15 +488,14 @@ def get_coords_icos_stations_atc():
         select ?station (GROUP_CONCAT(?piLname; separator=";") AS ?PI_names)
         where{
           ?station a cpmeta:AS .
-          ?pi cpmeta:hasMembership ?piMemb .
           ?piMemb cpmeta:atOrganization ?station  .
-           ?piMemb cpmeta:hasRole <http://meta.icos-cp.eu/resources/roles/PI> .
-           filter not exists {?piMemb cpmeta:hasEndTime []}
-           ?pi cpmeta:hasLastName ?piLname .
+          ?piMemb cpmeta:hasRole <http://meta.icos-cp.eu/resources/roles/PI> .
+          filter not exists {?piMemb cpmeta:hasEndTime []}
+          ?pi cpmeta:hasMembership ?piMemb .
+          ?pi cpmeta:hasLastName ?piLname .
         }
         group by ?station
         }
-        ?station a cpmeta:AS .
         ?station cpmeta:hasName ?stationName ;
            cpmeta:hasStationId ?stationId ;
            cpmeta:countryCode ?Country ;
@@ -581,8 +549,9 @@ def get_icos_stations_atc_L1():
               #prov:wasAssociatedWith/cpmeta:hasStationId ?stationId ;
               prov:startedAtTime ?timeStart ;
               prov:endedAtTime ?timeEnd ;
-              prov:wasAssociatedWith/cpmeta:hasName ?stationName 
+              prov:wasAssociatedWith ?stationIri 
             ] .
+            ?stationIri cpmeta:hasName ?stationName .
             ?dobj cpmeta:wasAcquiredBy/cpmeta:hasSamplingHeight ?height .
         }
         order by ?variable ?stationName ?height
