@@ -497,13 +497,18 @@ class Station():
 
 # --EOF Station Class-----------------------------------------            
 # ------------------------------------------------------------
-def get(stationId):
+def get(stationId, station_df=None):
     """
     Parameters
     ----------
     stationId : str StationId as you can extract with getIdList. Example 'NOR'
                 for Atmosphere Norunda station
-    example :  get('NOR')
+    Example :   get('NOR')
+
+    station_df: by default station_df is None.
+                However, if station_df is provided it is assumed it will be a
+                dataframe as generated by getIdList(). This is used by getList()
+                in order to increase performance
             
     Returns
     -------
@@ -514,8 +519,15 @@ def get(stationId):
     # create the station instance
     myStn = Station()
 
-    query = sparqls.getStations(stationId)
-    stn = RunSparql(query, 'pandas').run()
+    if isinstance(station_df, pd.DataFrame) and not station_df.empty:
+        try:
+            stn = station_df.loc['id' == stationId]
+        except:
+            query = sparqls.getStations(stationId)
+            stn = RunSparql(query, 'pandas').run()
+    else:
+        query = sparqls.getStations(stationId)
+        stn = RunSparql(query, 'pandas').run()
 
     if not isinstance(stn, pd.DataFrame) or stn.empty:
         myStn.stationId = stationId
@@ -559,17 +571,13 @@ def get(stationId):
     # flow from the thematic centres is achieved.        
 
     if 'ICOS' in myStn.project:
-        query = sparqls.stations_with_pi(stationId)
-        lstn = RunSparql(query, 'pandas').run()
-        if not lstn.empty:
-            myStn.theme = lstn['stationTheme'].values[0]
-            myStn.icosclass = lstn['class'].values[0]
-            myStn.icosclass = lstn['class'].values[0]
-            myStn.firstName = lstn['firstName'].values[0]
-            myStn.lastName = lstn['lastName'].values[0]
-            myStn.email = lstn['email'].values[0]
-            myStn.siteType = lstn['siteType'].values[0]
-            myStn.eag = lstn['eag'].values[0]
+        myStn.theme = stn.stationTheme.values[0].split('/')[-1]
+        myStn.icosclass = stn.icosClass.values[0]
+        myStn.firstName = stn.firstName.values[0]
+        myStn.lastName = stn.lastName.values[0]
+        myStn.email = stn.email.values[0]
+        myStn.siteType = stn.siteType.values[0]
+        myStn.eag = stn.elevation.values[0]
 
     # myStn._setData()
     return myStn
@@ -592,15 +600,16 @@ def getIdList(project='ICOS', sort='name', outfmt='pandas', icon=None):
 
     sort : str, optional
         The default is 'name'. A user can `sort` by any of the
-        dataframe's columns: uri, id, name, country, lat, lon,
-        elevation, project, theme.
+        dataframe's columns: uri, id, name, icosClass, country, lat, lon,
+                             elevation, stationTheme,
+                             firstName, lastName, email, siteType, project, theme.
 
     outfmt: str, optional
         The default is 'pandas'. If you provide 'map' to the `outfmt`
         argument a folium map is created with all known stations that
         have valid longitude and latitude values. Be advised that in
         this case, stations without a fixed location (like measurements
-        that belong to stations collected from instrumented Ships of
+        that belong to a station collected from instrumented Ships of
         Opportunity) will not be included.
 
     icon: None | str, optional
@@ -614,6 +623,8 @@ def getIdList(project='ICOS', sort='name', outfmt='pandas', icon=None):
     queried_stations : pandas.Dataframe
         `queried_stations` dataframe includes station id's, name,
         country, and landing page (or uri) among others.
+        (uri,id,name,icosClass,country,siteType,lat,lon,elevation,
+        stationTheme,firstName,lastName,email)
 
     stations_map : folium.Map
         `stations_map` is an interactive folium map with the available
@@ -635,9 +646,9 @@ def getIdList(project='ICOS', sort='name', outfmt='pandas', icon=None):
     project = project.upper()
     query = sparqls.getStations()
     queried_stations = RunSparql(query, 'pandas').run()
+
     queried_stations['project'] = queried_stations.apply(lambda x: __project(x['uri']), axis=1)
-    queried_stations['theme'] = queried_stations.apply(lambda x: x['uri'].
-                                                       split('/')[-1].split('_')[0], axis=1)
+    queried_stations['theme'] = queried_stations.apply(lambda x: x['stationTheme'].split('/')[-1], axis=1)
     if not project == 'ALL':
         queried_stations = queried_stations[queried_stations.project == project.upper()]
     # Drop any existing double entries.
@@ -707,7 +718,7 @@ def getList(theme=['AS', 'ES', 'OS'], ids=None):
     ----------
     theme : str | [iterable object of strings]) valid entries AS, ES, OS
     
-    stationIds : str | [iterable object of strings]) 
+    ids : str | [iterable object of strings])
     
     Returns
     -------
@@ -715,16 +726,16 @@ def getList(theme=['AS', 'ES', 'OS'], ids=None):
     
     """
 
-    stationList = []
+    station_list = []
 
     # if a list of id's is provided, ignore theme and class.    
     if ids:
         for s in tqdm(ids):
-            stationList.append(get(s))
+            station_list.append(get(s))
 
-        return stationList
+        return station_list
 
-    defaulttheme = ['AS', 'ES', 'OS']
+    default_theme = ['AS', 'ES', 'OS']
 
     # make sure input is a list and in uppercase    
 
@@ -732,21 +743,24 @@ def getList(theme=['AS', 'ES', 'OS'], ids=None):
         theme = [theme.upper()]
     elif isinstance(theme, list):
         theme = [x.upper() for x in theme]
-    if not isinstance(theme, list) or not (set(defaulttheme) & set(theme)):
+    if not isinstance(theme, list) or not (set(default_theme) & set(theme)):
         # looks like input is not a theme.
         # Revert to default values, return all certified stations.
-        theme = defaulttheme
+        theme = default_theme
 
-    # get station list, by default returns all icos stations
-    stations = getIdList()
-    # filter by theme
-    stations = stations[stations.theme.isin(theme)]
+    # Query stations.
+    # By default, `getIdList()` will return icos stations only.
+    stations_df = getIdList()
+    # Filter stations by theme and icos class if applicable.
+    stations_df = stations_df[stations_df.theme.isin(theme) &
+                              stations_df['icosClass'].isin(['1', '2', 'Associated'])]
 
-    stationList = []
-    for s in tqdm(stations.id):
-        stationList.append(get(s))
+    station_list = []
+    if isinstance(stations_df, pd.DataFrame) and (not stations_df.empty):
+        for s in tqdm(stations_df.id):
+            station_list.append(get(s, station_df=stations_df))
 
-    return stationList
+    return station_list
 
 
 # ------------------------------------------------------------    
@@ -772,5 +786,3 @@ if __name__ == "__main__":
     """
     print(msg)
 
-
-# ------------------------------------------------------------
