@@ -19,11 +19,8 @@ __date__        = "2021-11-04"
 
 import os
 import json
-import numpy as np
 import pandas as pd
-
 from tqdm.notebook import tqdm
-
 from icoscp.station import station as cpstation
 from icoscp.stilt.stiltobj import StiltStation
 from icoscp.stilt import geoinfo
@@ -86,7 +83,7 @@ def _outfmt(kwargs, stations):
         fmt = 'dict'
     
     # make sure we have a valid keyword, else set default dict
-    valid = ['dict', 'list', 'pandas', 'map']
+    valid = ['dict', 'list', 'pandas', 'map', 'avail']
     if not fmt in valid:
             fmt = 'dict'
     
@@ -105,6 +102,9 @@ def _outfmt(kwargs, stations):
     if fmt == 'map':
         return fmap.get(stations)
 
+    if fmt == 'avail':
+        return _avail(stations)
+
 def _country(kwargs, stations):
     """
     Find all stations belonging to a country based on information from
@@ -284,6 +284,58 @@ def _edate(kwargs, stations):
     return stations
 
 
+def _project(kwargs, stations):
+
+    proj = kwargs['project']
+    
+    # We use lowercase. 
+    proj = proj.lower()
+
+    flt = []
+    if proj == 'icos':
+        for k in stations:
+            if stations[k]['icos']:
+                flt.append(k)
+
+    stations =  {k: stations[k] for k in flt}
+    return stations
+
+
+def _avail(stations):
+    
+    years_set = set()
+    availability = {}
+
+    for stn in stations:
+        avail_per_stn ={}
+        if stations[stn]['icos']: 
+            avail_per_stn['ICOS id'] = stations[stn]['icos']['stationId']
+        else: 
+            avail_per_stn['ICOS id'] = ''
+        if stations[stn]['alt'] != None: 
+            avail_per_stn['Alt'] = stations[stn]['alt']
+        else: 
+            avail_per_stn['Alt'] = 0
+        if stations[stn]['years'] != None:
+            years_set = years_set.union(stations[stn]['years'])
+            for y in stations[stn]['years']:
+                avail_per_stn[int(y)] = int(stations[stn][y]['nmonths'])
+        availability[stn] = avail_per_stn
+    
+    year_list = list(range(min ({int(x) for x in years_set}),
+                              max ({int(x) for x in years_set}) + 1 ))
+    columns_list = ['Alt'] + year_list + ['ICOS id']
+      
+    df = pd.DataFrame(data = list(availability[x] for x in availability.keys()), 
+                      index = list(availability.keys()), 
+                      columns = columns_list)
+    # Fill in the gaps.
+    df[year_list] = df[year_list].fillna(0)
+    df[['Alt'] + year_list] = df[['Alt'] + year_list].applymap(int)
+
+    return df
+
+
 def _search(kwargs, stations):
     """ search for arbitrary string"""
     idx = []
@@ -299,8 +351,7 @@ def _search(kwargs, stations):
 
 
 def __get_object(stations):
-
-    return [StiltStation().get_info(stations[st]) for st in stations.keys()]
+    return [StiltStation(stations[st]) for st in stations.keys()]
 
 
 def __get_stations(ids=[], progress=True):
@@ -349,14 +400,14 @@ def __get_stations(ids=[], progress=True):
 
         loc_ident = os.readlink(CPC.STILTPATH+ist)
         clon = loc_ident[-13:-6]
-        lon = np.float(clon[:-1])
+        lon = float(clon[:-1])
         if clon[-1:] == 'W':
             lon = -lon
         clat = loc_ident[-20:-14]
-        lat = np.float(clat[:-1])
+        lat = float(clat[:-1])
         if clat[-1:] == 'S':
             lat = -lat
-        alt = np.int(loc_ident[-5:])
+        alt = int(loc_ident[-5:])
 
         stations[ist]['lat']=lat
         stations[ist]['lon']=lon
@@ -504,11 +555,15 @@ def find(**kwargs):
         Remember, that only year and month is checked.
     Example: station.find(dates=['2020-01-01', '2020/05/23'])
 
+    project STR 'icos'.
+        This will only return stilt stations that are ICOS stations.
+    Example: station.find(project='icos')
+        
     progress BOOL 
         By default progress is set to True, which returns a progressbar
         while searching the catalogue of STILT stations.    
     
-    outfmt STR ['dict'| 'pandas' | 'list' | 'map']:
+    outfmt STR ['dict'| 'pandas' | 'list' | 'map'| 'avail']:
         the result is returned as
         
             dict:       dictionary with full metadata for each station
@@ -517,6 +572,9 @@ def find(**kwargs):
             list:       list of stilt station objects
             map:        folium map, can be displayed directly in notebooks
                         or save to a static (leaflet) .html webpage
+            avail:      This choice will return availability of time series 
+                        data for all stilt stations. 
+                        Output format is a pandas dataframe.
     
     Returns
     -------
@@ -565,7 +623,8 @@ def find(**kwargs):
            'edate':_edate,
            'dates':_dates,
            'daterange':__daterange,
-           'search':_search}
+           'search':_search,
+           'project':_project}
 
     for k in kwargs.keys():
         if k in fun.keys():
@@ -650,4 +709,3 @@ def get(id=None, progress=False):
         return stationslist[0]
     else:
         return stationslist
-
